/*****************************************************************************
 * %LICENSE_DUAL%
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 *****************************************************************************/

/*****************************************************************************
 * Device IO control.
 *****************************************************************************/
int can_ioctl(
	can_os_t *can_os,
	unsigned int ctl_code,
	void *in,
	void *out
	)
{
	unsigned int err = 0;

	switch (ctl_code) {

	case ICP_CAN_IO_RESET:
		icp_can_set_run_mode(can_os->can, ICP_CAN_STOP);
		err = icp_can_reset(can_os);
		icp_can_set_run_mode(can_os->can, ICP_CAN_RUN);
		break;
	case ICP_CAN_IO_RUN:
		err = icp_can_set_run_mode(can_os->can, ICP_CAN_RUN);
		break;
	case ICP_CAN_IO_RUN_GET:
		err = icp_can_get_run_mode(can_os->can, (icp_can_run_mode_t *) out);
		break;			
	case ICP_CAN_IO_STOP:
		err = icp_can_set_run_mode(can_os->can, ICP_CAN_STOP);
		break;
	case ICP_CAN_IO_SIMPLE:
		icp_can_set_run_mode(can_os->can, ICP_CAN_STOP);
		err = icp_can_set_baud_simple(can_os->can, *((icp_can_baud_t *) in));
		icp_can_set_run_mode(can_os->can, ICP_CAN_RUN);
		break;
	case ICP_CAN_IO_CUSTOM:
		icp_can_set_run_mode(can_os->can, ICP_CAN_STOP);
		err = icp_can_set_baud_custom(can_os->can, (icp_can_timing_t *) in);
		icp_can_set_run_mode(can_os->can, ICP_CAN_RUN);
		break;
	case ICP_CAN_IO_TIMING_GET:		
		err = icp_can_get_baud(can_os->can, (icp_can_timing_t *) out);
		break;
	case ICP_CAN_IO_FILTER:	
		icp_can_set_rx_filter(can_os->can, (icp_can_rx_filter_t *) in);
		break;
	case ICP_CAN_IO_FILTER_GET:
		err = icp_can_get_rx_filter(can_os->can, (icp_can_rx_filter_t *) out);
		break;
	case ICP_CAN_IO_BLOCK:
		can_os->block_mode = 1;
		break;
	case ICP_CAN_IO_NON_BLOCK:
		can_os->block_mode = 0;
		break;
	case ICP_CAN_IO_BLOCK_GET:
		*((unsigned int *) out) = can_os->block_mode;
		break;
	case ICP_CAN_IO_LISTEN:
		err = icp_can_set_listen_mode(can_os->can, ICP_CAN_LISTEN);
		break;
	case ICP_CAN_IO_ACTIVE:
		err = icp_can_set_listen_mode(can_os->can, ICP_CAN_ACTIVE);
		break;
	case ICP_CAN_IO_LISTEN_GET:
		err = icp_can_get_listen_mode(can_os->can, (icp_can_listen_mode_t *) out);
		break;	
	case ICP_CAN_IO_ARBITER_ROUND_ROBIN:
		err = icp_can_set_arbiter_mode(can_os->can, ICP_CAN_ROUND_ROBIN);
		break;
	case ICP_CAN_IO_ARBITER_FIXED_PRIORITY:
		err = icp_can_set_arbiter_mode(can_os->can, ICP_CAN_FIXED_PRIORITY);
		break;
	case ICP_CAN_IO_ARBITER_GET:
		err = icp_can_get_arbiter_mode(can_os->can, (icp_can_arbiter_t *) out);
		break;
	case ICP_CAN_IO_ERROR_STATS_GET:
		err = icp_can_get_error_stats(can_os->can, (icp_can_error_t *) out);
		break;
	case ICP_CAN_IO_RESTART_MODE_AUTO:
		err = icp_can_set_restart_mode(can_os->can, ICP_CAN_AUTO);
		break;
	case ICP_CAN_IO_RESTART_MODE_MANUAL:
		err = icp_can_set_restart_mode(can_os->can, ICP_CAN_MANUAL);
		break;
	case ICP_CAN_IO_RESTART_MODE_GET:
		err = icp_can_get_restart_mode(can_os->can, (icp_can_auto_restart_t *) out);
		break;
	case ICP_CAN_IO_BUFFER_LINK_SET:
		err = icp_can_set_rx_buffer_link(can_os->can, *((int *) in), 1);
		break;
	case ICP_CAN_IO_BUFFER_LINK_CLEAR:
		err = icp_can_set_rx_buffer_link(can_os->can, *((int *) in), 0);
		break;
	case ICP_CAN_IO_BUFFER_LINK_GET:
		err = icp_can_get_rx_buffer_link(can_os->can, *((int *) in), (unsigned int *) out);
		break;
	case ICP_CAN_IO_RX_ENABLE_SET:
		err = icp_can_set_rx_enable(can_os->can, *((int *) in), 1);
		break;
	case ICP_CAN_IO_RX_ENABLE_CLEAR:
		err = icp_can_set_rx_enable(can_os->can, *((int *) in), 0);
		break;
	case ICP_CAN_IO_RX_ENABLE_GET:
		err = icp_can_get_rx_enable(can_os->can, *((int *) in), (unsigned int *) out);
		break;
	case ICP_CAN_IO_TX_ENABLE_SET:
		err = icp_can_set_tx_enable(can_os->can, *((int *) in), 1);
		break;
	case ICP_CAN_IO_TX_ENABLE_CLEAR:
		err = icp_can_set_tx_enable(can_os->can, *((int *) in), 0);
		break;
	case ICP_CAN_IO_TX_ENABLE_GET:
		err = icp_can_get_tx_enable(can_os->can, *((int *) in), (unsigned int *) out);
		break;
	default:
		CAN_PRINT_DEBUG("Unrecognizined IOCTL, skipping 0x%x. \n", ctl_code);
		break;
	}

	return err;
}

void can_ioctl_get_size(
	unsigned int ctl_code,
	size_t *in_size,
	size_t *out_size
	)
{
	*in_size = 0;
	*out_size = 0;

	switch (ctl_code) {

	case ICP_CAN_IO_SIMPLE:
		*in_size = sizeof(icp_can_baud_t);
		break;
	case ICP_CAN_IO_CUSTOM:
		*in_size = sizeof(icp_can_timing_t);
		break;
	case ICP_CAN_IO_FILTER:	
		*in_size = sizeof(icp_can_rx_filter_t);
		break;
	case ICP_CAN_IO_RUN_GET:
		*out_size = sizeof(icp_can_run_mode_t);
		break;
	case ICP_CAN_IO_TIMING_GET:
		*out_size = sizeof(icp_can_timing_t);
		break;
	case ICP_CAN_IO_FILTER_GET:
		*out_size = sizeof(icp_can_rx_filter_t);
		break;
	case ICP_CAN_IO_BLOCK_GET:
		*out_size = sizeof(unsigned int);
		break;
	case ICP_CAN_IO_LISTEN_GET:
		*out_size = sizeof(icp_can_listen_mode_t);
		break;
	case ICP_CAN_IO_ARBITER_GET:
		*out_size = sizeof(icp_can_arbiter_t);
		break;
	case ICP_CAN_IO_ERROR_STATS_GET:
		*out_size = sizeof(icp_can_error_t);
		break;
	case ICP_CAN_IO_RESTART_MODE_GET:
		*out_size = sizeof(icp_can_auto_restart_t);
		break;
	case ICP_CAN_IO_BUFFER_LINK_SET:
	case ICP_CAN_IO_BUFFER_LINK_CLEAR:
	case ICP_CAN_IO_RX_ENABLE_SET:
	case ICP_CAN_IO_RX_ENABLE_CLEAR:
	case ICP_CAN_IO_TX_ENABLE_SET:
	case ICP_CAN_IO_TX_ENABLE_CLEAR:
		*in_size = sizeof(unsigned int);
		break;
	case ICP_CAN_IO_BUFFER_LINK_GET:
	case ICP_CAN_IO_RX_ENABLE_GET:
	case ICP_CAN_IO_TX_ENABLE_GET:
		*in_size = sizeof(unsigned int);
		*out_size = sizeof(unsigned int);
		break;
	default:
		break;
	}
}
