/*****************************************************************************
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 *****************************************************************************/

/**
 *****************************************************************************
 * @defgroup EDMA_GENERAL Base types, #defines, and memory mapped register
 * offsets for EDMA module.
 *      File:  dma.h
 * 
 * @description
 *      This is a group of related structures related to internal workings of
 *      the driver. It includes EDMA transfer modes, register offset values,
 *      memory mapped register and associated data structures and valid values,
 *      return types, etc.  
 * 
 *****************************************************************************/

/*****************************************************************************
 *
 * Contents:
 *
 *      The following data structures are exported by the API
 *                  
 *      edma_cookie_t: a unique value associated with each tranaction.
 *      edma_status_t: the status that is returned from an edma_is_complete
 *                     call.
 *      edma_channel_priority_status_t: status returned from a call 
 *                                      to edma_change_channel_priority.
 *      edma_callback_t: the prototype for a client driver's callback function.
 *      edma_client: a struct that maintains client information.
 *
 *****************************************************************************/

/*****************************************************************************/

#ifndef DMA_H
#define DMA_H

/*****************************************************************************/

#ifdef __FreeBSD__

    #include <sys/ioccom.h> /* for _IOR, IOW, _IO, etc */
    #include <sys/types.h>
    #include <sys/lock.h>
#endif
#ifdef __KERNEL__

    #include <linux/init.h>
    #include <linux/types.h>
    
    
#endif
/*
 *  Platform dependent values. 
 */
#include "os.h"
#include "os_list.h"



#ifdef EDMA_TOL

    #define EDMA_DEVICE_ID              0x5023
    #define PRODUCT_STRING             "Enhanced DMA Driver"
    #define CHAR_DRV_NAME              "edma_01"
    
#endif
    
#ifdef EDMA_WML

    #define EDMA_DEVICE_ID              0x35B5
    #define PRODUCT_STRING             "EDMA Reference Driver - CPL"
    #define CHAR_DRV_NAME              "edma_01"
   
#endif


#define ALIGN_FOR_EDMA(x) __attribute__((__aligned__(x)))
#define MAX_TRANSFER_PER_DESC       0x01000000  /*  16MB */
/*
 * Memory Operations: Transfer Modes for Source and Destination
 * Functionality present in EDMA_W and EDMA_TOL platform.
 * Bits 9-11 Reserved.
 */
#define SRC_INC         0x00000000 /**< @ingroup EDMA_GENERAL
                                    *Source Addr in
                                    *Increment Transfer Mode*/
#define SRC_DEC         0x00001000 /**< @ingroup EDMA_GENERAL
									*Source Addr in
                                    *Decrement Transfer Mode*/
#define SRC_BUFFER_INIT 0x00002000 /**< @ingroup EDMA_GENERAL
									*Source Addr in
                                    *Buffer Initiazation Transfer Mode */

#define DST_INC         0x00000000 /**< @ingroup EDMA_GENERAL
									*Destination Addr in
                                    *Increment Transfer Mode*/
#define DST_CONST       0x00004000 /**< @ingroup EDMA_GENERAL
									*Destination Addr in
                                    *Constant Transfer Mode*/

/* Bit 16 reserved */

#define GRAN_1_BYTE     0x00000000 /**< @ingroup EDMA_GENERAL
                                    *Destination Addr in
                                    *1 Byte Transfer Mode */
#define GRAN_2_BYTE     0x00020000 /**< @ingroup EDMA_GENERAL
									*Destination Addr in
                                    *2 Byte Transfer Mode*/
#define GRAN_4_BYTE     0x00040000 /**< @ingroup EDMA_GENERAL
									*Destination Addr in
                                    *4 Byte Transfer Mode*/

/* Bits 19-28 Reserved */

/* Traffic Classes */
#define TC_0            0x00000000 /**< @ingroup EDMA_GENERAL
                                   * Traffic class 0 */
#define TC_1            0x20000000 /**< @ingroup EDMA_GENERAL
                                   * Traffic class 1 */
#define TC_2            0x40000000 /**< @ingroup EDMA_GENERAL
                                   * Traffic class 2 */
#define TC_3            0x60000000 /**< @ingroup EDMA_GENERAL
                                   * Traffic class 3 */
#define TC_4            0x80000000 /**< @ingroup EDMA_GENERAL
                                   * Traffic class 4 */
#define TC_5            0xA0000000 /**< @ingroup EDMA_GENERAL
                                   * Traffic class 5 */
#define TC_6            0xC0000000 /**< @ingroup EDMA_GENERAL
                                   * Traffic class 6 */
#define TC_7            0xE0000000 /**< @ingroup EDMA_GENERAL
                                   * Traffic class 7 */

/* Bit 5: Destination Address Points to local system memory or I/O Space */
#define DST_MEM   0x00000000 /**< @ingroup EDMA_GENERAL
                               *Destination Address points to a
                               *memory mapped region */
#define DST_IO 	  0x00000020 /**< @ingroup EDMA_GENERAL
							   *Destination Address points to a
                               *memory mapped I/O region*/

/* Bit 6 - Source address points to local system memory or I/O Space.
Should only be zero for this implementation */
#define SRC_MEM	  0x00000000  /**< @ingroup EDMA_GENERAL
                               *Source Address points to a
                               *memory mapped region*/
#define SRC_IO	  0x00000040 /**< @ingroup EDMA_GENERAL
							   *Destination Address points to a
                               *memory mapped I/O region*/

/* Bit 7 - Destination is non-coherent or coherent */
#define DST_NONCOHERENT 0x00000000  /**< @ingroup EDMA_GENERAL
                                    *Destination Address points to a
                                    *non-coherent region, 
									*no FSB snooping */
#define DST_COHERENT    0x00000080  /**< @ingroup EDMA_GENERAL
									*Destination Address points to a
                                    *coherent region */

/* Bit 8 - Source is non-coherent or coherent */
#define SRC_NONCOHERENT 0x00000000   /**< @ingroup EDMA_GENERAL
                                     *Source Address points to a
                                     *non-coherent region, 
									 *no FSB snooping */
#define SRC_COHERENT    0x00000100   /**< @ingroup EDMA_GENERAL
									 *Source Address points to a
                                     *coherent region */

/* Bits 9  Source Alignment */

#define SRC_DWORD_ALIGNED 0x00000200 /**< @ingroup EDMA_GENERAL
                                      *Source Address points to a
                                      *DWORD aligned memory region*/
#define SRC_CACHE_ALIGNED 0x00000000 /**< @ingroup EDMA_GENERAL
									  *Source Address points to a
                                      *cache aligned memory region*/

/* Bits 10  Destination Alignment */
#define DST_DWORD_ALIGNED 0x00000400 /**< @ingroup EDMA_GENERAL
                                      *Destination Address points to a
                                      *DWORD aligned memory region*/
#define DST_CACHE_ALIGNED 0x00000000 /**< @ingroup EDMA_GENERAL
									  *Destination Address points to a
                                      *cache aligned memory region*/
                                      
#define ADDR_MODE_ALIGNED_COHERENT (SRC_DWORD_ALIGNED | DST_DWORD_ALIGNED\
| DST_COHERENT | SRC_COHERENT)

#define DEBUG 1
/*
*  Macros
*/
#define resource_to_dma(res) (struct edma_chan *)((res)->ptr)

#define BYTE   1 
#define WORD   2
#define DWORD  4


/* PCI CONFIGURATION -
*  Configuration Space Register Offsets
*/
#define EDMA_PCICMD       0x04   /* PCI Command Reg Size = 16 bits */
#define EDMA_CTL_REG	  0x40	 /* PCI Control Size = 8 bits  */
#define EDMA_FERR_REG	  0x80   /* PCI Error Reg Size = 32 bits */


/* MEMORY MAPPED REGISTERS - 
*  Offsets for the bank of memory registers associate with
*  each channel.
*/

#define EDMA_CHAN0_CCR  0x00   /* Channel 0 Size = 32 bits */
#define EDMA_CHAN1_CCR  0x40   /* Channel 1 Size = 32 bits */
#define EDMA_CHAN2_CCR  0x80   /* Channel 2 Size = 32 bits */
#define EDMA_CHAN3_CCR  0xC0   /* Channel 3 Size = 32 bits */

#define EDMA_CCR        0x00 /* Control Register, Size = 32 bits */
#define EDMA_CSR        0x04 /* Status Register, Size = 32 bits */
#define EDMA_CDAR       0x08 /* Current Descriptor Register
                                Adddress, Size = 32 bits */
#define EDMA_CDUAR      0x0C /* Current Descriptor Register
                                Upper Address, Size = 32 bits */
#define EDMA_SAR	0x10 /* Source Register
                                Address, Size = 32 bits */
#define EDMA_SUAR       0x14 /* Source Register
                                Upper Address, Size = 32 bits */
#define EDMA_DAR        0x18 /* Destination Register
                                Address, Size = 32 bits */
#define EDMA_DUAR       0x1C /* Destination Register
                                Upper Address, Size = 32 bits */
#define EDMA_NDAR	0x20 /* Next Descriptor Register
                                Address, Size = 32 bits */
#define EDMA_NDUAR      0x24 /* Next Descriptor Register
                                Upper Address, Size = 32 bits */
#define EDMA_TCR	0x28 /* Byte Count Register,
                                Size = 32 bits */
#define EDMA_DCR        0x2C /* Command Register,
                                Size = 32 bits */



/* MEMORY MAPPED REGISTERS - 
*  Global Control Registers
*/
#define EDMA_DCGC_REG               0x100      /* Size = 32 bits */

/* VALUES*/
#define EDMA_VENDOR_ID              0x8086

#define EDMA_CTL_ENABLE             0x80  /* Control Register enable */
//#define EDMA_CTL_ENABLE             0x88  /* Control Register enable */
#define EDMA_FERR_RESET             0xFFFFFFFF /* Write all ones to clear FERR */
#define EDMA_BAR0                   0x00000000
#define MAJOR_NO                   55
#define SUCCESS                    0
#define FAILED                     -1
#define CHANNEL_PRIORITY_ENABLED   0x04
#define CHANNEL_PRIORITY_DISABLED  0xFFFFFFF8
#define EDMA_CMD_VAL               0x02

/* Values for checking a channel  status register */
#define END_OF_CHAIN      0x01 /* End of Chain Event Occurred */
#define END_OF_TRANSFER   0x02 /* End of Transfer Event Occurred */
#define DMA_SUSPENDED     0x04 /* Suspension of Transfer Occurred */
#define DMA_STOPPED       0x08 /* DMA Engine has stopped */
#define DMA_ABORTED       0x10 /* An abort event occurred */
#define CHAN_ACTIVE       0x20 /* A channel is actively engaged in a memory
                                  transfer operation */
#define CSR_IDLE          0x00000000 /**< CSR value when
                                       system powered on or reset */


/* Values for writing to a channel's command register */
#define START_DMA    0x01   /* Start DMA Transfer */
#define SUSPEND_DMA  0x02   /* Suspend an active DMA Transfer */
#define STOP_DMA     0x04   /* Use Carefully!
                               See documentation for details.*/
#define RESUME_DMA   0x08   /* Resume a transfer after a supsension */

#define FERR_ERR0    0x00
#define FERR_ERR1    0x01
#define FERR_ERR2    0x02
#define FERR_ERR3    0x03
#define FERR_ERR4    0x04
#define FERR_ERR5    0x05
#define FERR_ERR6    0x06
#define FERR_ERR7    0x07


/* Defines for the DWORD Memory_Ops which are placed in the DCR1 */
#define CLEAR_MEMORY_OPS 0xFFFFFFE0
/* Bits 0 - 4 are not bits that can be set by the user.
We set END_OF_CHAIN as a driver policy, and we won't allow 
the user to set these bits. 
*/
#define INT_END_OF_CHAIN     0x00000001 /* Interrupt occurs on an
                                           end of chain event.
                                            */
#define INT_END_OF_TRANSFER  0x00000002 /* Interrupt occurs on an
                                           end of transfer event.
                                            */
#define INT_SUSPEND	     0x00000004 /* Interrupt occurs on an
                                           suspension event.
                                            */
#define INT_STOPPED	     0x00000008 /* Interrupt occurs when
                                           DMA engine has stopped.
                                            */
#define INT_ABORT	     0x00000010 /* Interrupt occurs on
                                           an abort event.
                                            */

#define SHIFT_DWORD  32
#define PM_DELAY     20000
#define F_PAGE       4096
#define HANDLE_NO    2
#define GET_CHANNEL  2
#define EDMA_PERMISSIONS 0600

/* IOCTL VALUES - 
*  The "magic number" is necessary for the "new" way of
*  choosing ioctl numbers as described in Documentation/ioctl-number.txt,
*  which outlines how to add new ioctls to the kernel using
*  the _IO macros.
*/


/* STRINGS -
*  String Constants
*
*/




/* MESSAGES Linux, Common */
 /*****************************************************************************/
#define ERROR_REGISTRATION         "EDMA: Init - could not register module\n"
#define ERROR_DEREGISTRATION	   "EDMA: Cleanup - could not unregister \
module\n"	
#define MESSAGE_INIT               "EDMA: Init - Performing initialization\n"
#define CHAR_DVR_REG_FAILED        "EDMA: Init - Character driver \
initialization failed\n"
#define CHAR_DEVICE_ALLOC          "EDMA: Unable to add Character Device %x\n"
#define MESSAGE_INIT_COMPLETE      "EDMA: Init - Initialization complete\n"
#define ERROR_MM1                  "EDMA: Init - Failed to obtain \
PCI I/O Memory Region...\n"
#define ERROR_MM2                  "EDMA: Init- Failed to obtain a \
virtual address for memory mapped registers\n"

#define MESSAGE_SUSPEND            "EDMA: Entering edma_suspend \n"
#define MESSAGE_RESUME             "EDMA: Entering edma_resume \n"

#define ERROR_DEVICE_NOT_FOUND     "EDMA: Init - Failed to find device\n"
#define MESSAGE_CLEANUP            "EDMA: Cleanup - Performing Cleanup\n"
#define MESSAGE_DEREGISTRATION     "EDMA: Deregistration successful. \
Cleanup complete\n"
#define MESSAGE_CHANNEL            "EDMA: Channel Allocation broken\n"
#define ERROR_CLIENT_ALLOC         "EDMA: Unable to allocate an \
edma_client struct\n"
#define UNKNOWN_IOCTL              "EDMA: Unknown IOCTL operation\n"
#define CHANNEL_ACTIVE             "EDMA: Channel Active\n"
#define CHANNEL_IDLE               "EDMA: Channel Idle\n"
#define CREATING_INTERNAL_STRUCT   "EDMA: Creating Internal Structures\n" 
#define ERROR_INTERNAL_STRUCT      "EDMA: Unable to create \
internal structures\n"
#define ERROR_INIT_EDMA_CTL_REG    "EDMA: Couldn't initialize\
 EDMA_CTL_REG\n"
#define ERROR_INIT_EDMA_FERR_REG   "EDMA: Couldn't initialize\
 EDMA_FERR_REG\n"
#define ERROR_INIT_PCICMD          "EDMA: Couldn't initialize \
EDMA_PCICMD\n"
#define ERROR_MSI_SUPPORT          "EDMA: MSI support not \
available: %i.\n"
#define ERROR_MSI_VECTOR           "EDMA: MSI Interrupt \
vector: %i.\n"
#define ERROR_PCI_REGISTRATION     "EDMA: Error on PCI \
registration.\n"
#define EDMA_ABORTED_FERR_VAL      "EDMA: EDMA_ABORTED FERR \
Value %x \n"
#define EDMA_END_OF_CHAIN_VAL      "EDMA: END_OF_CHAIN Value \
%x \n"
#define MESSAGE_PROBE              "EDMA: Probing for EDMA \
device\n"
#define MESSAGE_REMOVE             "EDMA: Unregistering EDMA \
device\n"








/* MESSAGES FreeBSD */
#define DMA_SUCCESSFUL_OPEN           "EDMA:: Opening Driver\n"
#define DMA_SUCCESSFUL_CLOSE          "EDMA: Closing Driver\n"
#define DMA_PROBE                     "EDMA: REACHED PROBE %x %x\n"
#define DMA_FOUND_DEVICE              "EDMA: FOUND DEVICE: %x\n"
#define DMA_ATTACH                    "EDMA: Attach\n"
#define DMA_DETACH                    "EDMA: Detaching\n"


#define DMA_ATTACH_COMPLETE           "EDMA: Attach \
complete\n"
#define DMA_ALLOC_INTERNAL            "DMA_DRIVER %s:Couldn't \
allocate \
structures properly\n"
#define DMA_DEV_ENTRY                 "dma-driver"
#define DMA_CONFIG_REGS               "EDMA: Writing to configuration \
registers...\n"
#define DMA_PCIBAR                    "EDMA:Failed to allocate pci \
bar resource\n"
#define DMA_PCIBAR_RESOURCE_RELEASE   "Results of releasing \
resource %x\n"
#define DMA_IRQ                       "EDMA:Failed to allocate \
irq resource\n"
#define DMA_IRQ_HANDLER               "EDMA:Couldn't set up irq handler: \
error is %i \n"
#define DMA_CHAR_DEVICE               "EDMA:Failed to allocate device \
structure\n"
#define DMA_IRQ_ID                    0x00


#define DMA_SOFT_SIZE 24

/* MISC */
#ifdef __FreeBSD__

    #define TRUE                       1
    #define FALSE                      0
    
#endif
#define OBSOLETE 1 
#define CURRENT 0
#define MAX_NO_CHANS               4   /* maximum number of channels */
#define NOT_IN_USE                 0
#define IN_USE			   1
#define DWORD_ALIGNED		   0x20
#define DEFAULT_DESCRIPTOR_NO      64
#define LOWER_64                   0x00000000FFFFFFFF
#define UPPER_64                   0xFFFFFFFF00000000
#define INVALID_COOKIE             -1
#define BLOCK_NOT_FOUND            -1
#define BLOCK_FOUND                 1

#define ERROR_OUT_OF_DESCRIPTORS   -1
#define ERROR_BAD_CHANNEL          -2
#define EXCEED_LIMIT                100 /* number of u periods to wait */
#define ERROR_SUSPEND_DMA          -1
#define PERMISSIONS                 0600

#ifdef DMA_DEBUG_PRINT 
#define DMA_DEBUG_PRINT_OS_0(S)        os_print_debug(S) 
#define DMA_DEBUG_PRINT_OS_1(S, A)     os_print_debug(S,A)
#define DMA_DEBUG_PRINT_OS_2(S, A, B)  os_print_debug(S,A,B)
#else
#define DMA_DEBUG_PRINT_OS_0(S) 
#define DMA_DEBUG_PRINT_OS_1(S, A)   
#define DMA_DEBUG_PRINT_OS_2(S, A, B)
#endif

/* TEST SIZES */
#define UNIT_TEST_SIZE 4096
#define ICU_TEST_SIZE 4096
/* These references are here to prevent unnecessary warnings. */
struct edma_chan;
struct edma_client;

/**
*****************************************************************************
* @ingroup EDMA_GENERAL
* A cookie value type.
* @description
* This type is returned by edma_memcpy for each transaction request.
* The calling module (client module) may use the cookie value, which is
* associated with a memory transaction, to query the state of the transaction.
*****************************************************************************/
typedef signed long edma_cookie_t;

/******************************************************************************
* Abstract
* Memory mapped registers associated with each channel.
* 
* Purpose:
* The structure is used internally by the EDMA driver to
* write memory mapped register values.
* The remaining elements are described in the descriptor
* structure below.
******************************************************************************/
struct edma_ch_mm_regs
{
    uint32_t ccr;   /* Channel control register */
    uint32_t csr;   /* Channel status register */
    uint32_t cdar;  /* Channel current desc register */
    uint32_t cduar; /* Channel current desc upper register */
    uint32_t sar;   /* Channel source address register */
    uint32_t suar;  /* Channel source address upper register */
    uint32_t dar;   /* Channel destination register */
    uint32_t duar;  /* Channel destination upper register */
    uint32_t ndar;  /* Channel next desc register */
    uint32_t nduar; /* Channel next desc upper register */
    uint32_t tcr;   /* Channel transaction count register (in bytes) */
    uint32_t dcr;   /* Descriptor control register */
   /* There are 16 bytes between each set of per-channel registers
    * as well as sixteen bytes between the channel 3's registers
    * and the global control registers. 
    */
    uint32_t reserved[4]; /* Reserved - unused */

};



/******************************************************************************
* Abstract
* Registers comprising a "descriptor."
* Purpose:
* Each channel has n number descriptors associated with it. The descriptors
* are chained together in a linked list. Each descriptor describes a memory
* copy operation. 
******************************************************************************/
struct edma_descriptor
{
    uint32_t sar;   /* Source address */
    uint32_t suar;  /* Source upper address */
    uint32_t dar;   /* Destination address */
    uint32_t duar;  /* Destination upper address */
    uint32_t ndar;  /* Next descriptor address */
    uint32_t nduar; /* Next descriptor upper address */
    uint32_t tcr;   /* Number of Bytes Count */
    uint32_t dcr;   /* Descriptor Control */

};

/******************************************************************************
* Abstract
* Memmory mapped register sets for channels 0 - 3
* Purpose:
* Each channel has a group of memory mapped registers associated with it.
* The elements of this structure point to each channel's group of registers
* as wells as global registers associated with all of the channels.
******************************************************************************/
struct edma_mm_regs
{
    struct edma_ch_mm_regs ch0;/* memmory mapped registers for channels 0 - 3*/
    struct edma_ch_mm_regs ch1;
    struct edma_ch_mm_regs ch2;
    struct edma_ch_mm_regs ch3;
    uint32_t dcgc; /* DMA Controller Global Command */
    uint32_t dcgs; /* DMA Controller Global Status */
};


/*
 * Return values for edma_is_complete function.
 */

/**
 *****************************************************************************
 * @ingroup EDMA_GENERAL
 * Return values for edma_is_complete function.
 * @description
 * The edma_is_complete function uses this type to return
 * status about queries on a particular transaction.
 *****************************************************************************/
enum edma_status_t
{
    EDMA_NO_RECORD = -1, /**< No record for transaction is available
                          cookie expired. */
    EDMA_SUCCESS,     /**< Transaction completed successfully */
    EDMA_IN_PROGRESS, /**< Transaction in progress */
    EDMA_ERROR        /**< Error occurred in transaction */
    	                           
};

/**
 *****************************************************************************
 * @ingroup EDMA_GENERAL
 * Return values for edma_callback_t function.
 * @description
 * The edma_is_complete function uses this type to return
 * status about queries on a particular transaction.
 *****************************************************************************/
enum edma_event_t
{
    EDMA_RESOURCE_SUSPEND,  /**< Power Management Event */
    EDMA_RESOURCE_RESUME,   /**< Power Management Event */
    EDMA_RESOURCE_ADDED,    /**< Channel Resource Granted */
    EDMA_RESOURCE_REMOVED,     /**< Channel Resource Taken Away */
    
   
};

/**
 *****************************************************************************
 * @ingroup EDMA_GENERAL
 * Return values for edma_callback_t function.
 * @description
 * The edma_is_complete function uses this type to return
 * status about queries on a particular transaction.
 *****************************************************************************/
enum edma_completion_event_t
{
    EDMA_TRANSACTION_COMPLETE, /**< Transaction Completed without Error */
    EDMA_TRANSACTION_ERROR     /**< Transaction Completed with Error */
};

/**
 *****************************************************************************
 * @ingroup EDMA_GENERAL
 * Return values for the edma_change_channel_priority.
 * @description
 * The edma_change_channel_priority function uses this type to return
 * status about the request to change channel priority.
 *****************************************************************************/
enum edma_channel_priority_status_t
{
    EDMA_CP_FAIL = -1,	/**< Failed to reset channel priority */
    EDMA_CP_DEFAULT,    /**< Set channel priority to default (round robin)*/
    EDMA_CP_VALID_CHANNEL /**< Set user specified channel to have top priority*/
};

 /*****************************************************************************
 * Abstract: 
 * This enumerated type leaves room for expansion - the
 * inclusion of other types.
 * Purpose:
 *
 * Fields:
 *  
 *****************************************************************************/
enum edma_resource_t {
    EDMA_RESOURCE
};

 /*****************************************************************************
 * Abstract: 
 * Contains the resource information per client.
 * Purpose:
 * This is a simple structure to hold resource information on a per client 
 * basis.
 * Fields:
 *     type Type of Resource
 *     ptr  
 *****************************************************************************/
struct edma_resource {
    enum edma_resource_t type;
    void *ptr;
};

/*
 * TYPEDEF: edma_callback_t - client drivers should implement
 * a callback function with a prototype which mirrors the
 * parameters below.
 */

/**
 *****************************************************************************
 * @ingroup EDMA_GENERAL
 * A type definition  for callback routines.
 * @description
 * The driver use "callbacks" to relay status and information
 * to the the client driver. Client drivers need to define their
 * callback routine prototypes to conform like the function prototype below.
 *****************************************************************************/
typedef void  (*edma_callback_t ) (struct edma_client *, 
/**< ptr to a client structure*/
struct edma_chan *, /**< pointer to a channel structure */
enum edma_event_t);/**< type of event */
                                   

/**
 *****************************************************************************
 * @ingroup EDMA_GENERAL
 * A type definition  for callback routines.
 * @description
 * The driver use "callbacks" to relay status and information
 * to the the client driver. Client drivers need to define their
 * callback routine prototypes to conform like the function prototype below.
 *****************************************************************************/
typedef void  (*edma_completion_callback_t ) (struct edma_client *, 
/**< ptr to a client structure*/
struct edma_chan *, /**< pointer to a channel structure */
enum edma_completion_event_t, /**< type of event */
edma_cookie_t); /**< cookie associated with certain events*/

/**
 *****************************************************************************
 * @ingroup EDMA_GENERAL
 * A client structure 
 * @description
 * The client structure holds information about a client module that will
 * attempt to register with the driver. It is allocated by the EDMA driver.
 * Client drivers allocate a client struct with the Client API.
 *****************************************************************************/
struct edma_client {
    uint32_t chan_count; /**< current number of channels granted >**/
    uint32_t chan_desired; /**< number of channels requested >**/
    edma_callback_t edma_callback; /**< pointer to a callback function >**/
    edma_completion_callback_t edma_completion_callback; 
    /**< pointer to a callback function >**/
    int32_t channels[4];        /**< list of granted channels >**/
    struct os_list_start global_node; /**< list of registered clients >**/
    
};




#endif



