/*****************************************************************************
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 *****************************************************************************/

/**
 *****************************************************************************
 * @defgroup EDMA Internal 
 * EDMA Internal Data Structures
 * File:  dma_internals.h
 * $Revision: $
 * @purpose
 *   This is a group of related structures related to internal driver book
 *   keeping and lists, such as lists of clients requesting channels, 
 *   driver descriptors, data associated with channels, etc.
 *****************************************************************************/

#ifndef DMA_INTERNALS_H
#define DMA_INTERNALS_H


#include "dma.h"


#ifdef __FreeBSD__
#endif

#ifdef __KERNEL__
    #include <linux/init.h>
    #include <linux/pci.h>
    #include <linux/types.h>
    #include <linux/dmaengine.h>
    
    /**
    *****************************************************************************
    * @ingroup EDMA_GENERAL
    * A cookie value type.
    * @description
    * This type is returned by edma_memcpy for each transaction request.
    * The calling module (client module) may use the cookie value, which is
    * associated with a memory transaction, to query the state of the transaction.
    *****************************************************************************/


    /* FOR LINUX I/OAT */
struct edma_device
{
    struct pci_dev * pci_dev;
    uint32_t pci_cfg_space[64];
    void *reg_base;
    struct dma_device common;

};

    
#endif

 
typedef uint64_t edma_addr_t; 



/******************************************************************************
* Abstract
* Channel representing a DMA Channel and associated data.
* Purpose:

******************************************************************************/
struct edma_chan
{
    uint8_t * next_ptr;
    uint8_t * prev_ptr;
   
    uint32_t channel_no; /* channels are organized by number 0...3 */
    uint32_t reg_offset; /* private reserved */
    int32_t sw_in_use; /* value set when a channel is allocated, 
                          1 = in use, 0 = not in use */
    
    edma_cookie_t completed_cookie; /* completed_cookie last 
                                    cookie seen completed on cleanup */       
    edma_cookie_t cookie; /* value of last cookie given to client */          
    uint32_t memory_ops;
    struct edma_client *client;
    os_spinlock_t desc_lock; /* used for spin lock operations */

    
    
};


/******************************************************************************
* Abstract
* A wrapper structure around the edma_client.
* Purpose:

******************************************************************************/
struct edma_client_internal {
    uint8_t * next_ptr;/* points to next edma_client_internal in list */
    uint8_t * prev_ptr;/* points to previous edma_client_internal in list */
    struct edma_client pub; /* public dma client */  
};

/******************************************************************************
* Abstract
* a wrapper structure around the edma_descriptor.
* DEFAULT_DESCRIPTOR_NO (n number) descriptors are created and formatted
* into a linked list (1 list/channel).
* Purpose:

******************************************************************************/
struct edma_descriptor_internal
{
    struct edma_descriptor hw; /* descriptor struct (must be dword aligned) */
    uint32_t next_ndar; /* points to next descriptor (lower add) */
    uint32_t next_nduar; /* points to next descriptor (upper add) */
    edma_cookie_t cookie; /* cookie assigned to the descriptor */
    edma_addr_t phys; /* physical address of the descriptor */
    struct edma_descriptor_block * block;
    int32_t error; 
    uint32_t reserved; /* reserved for expansion */
};


/******************************************************************************
* Abstract
* Struct containing the descriptor list information. 
* (1 descriptor list/channel)
* 
* Purpose:
*
* The descriptor indices are an internal mechanism created
* to help manage and track which descriptors are
* currently in use and which are available. The descriptor_pool points
* to n number descriptors kernel malloced
******************************************************************************/

struct edma_descriptor_block
{
    
    uint8_t * next_ptr;    /* points to next block in list */
    
    uint8_t *  prev_ptr;   /* points to previous block in list */
    struct edma_descriptor_internal * descriptor_pool; 
                           /* points to a contiguous region of memory
    set aside for n descriptors, may be adjusted for alignment purposes */
    ALIGN_FOR_EDMA(32) uint8_t * pool_addr; 
    
                          /* points to a contiguous region of memory
    set aside for n descriptors, original unadjusted addr */
    uint32_t handle;
    int32_t free_index;   /* index of the next free buffer */
    int32_t used_index;   /* index of the first used descriptor */
    uint32_t status;      /* OBSOLETE (1) = ready to be deleted from a list, 
                          CURRENT (0) = being used */
    uint32_t last_cookie;    
    /* Last cookie value associated with the block */
    uint32_t last_ferr_value;   /* last FERR value */
    uint32_t last_cookie_err;  /* cookie associated with the FERR value */
    uint32_t last_desc_err;    /* descriptor associated with the value */
    
};

struct edma_descriptor_list
{
    struct os_list_start block_list; /* list of descriptor blocks */
    struct os_list_start del_list; /* list of descriptor blocks */
    uint32_t block_count;
};



uint8_t * get_mm_registers(uint32_t channel, uint32_t offset);
int32_t edma_suspend_device(void);
void edma_resume_device(void);

struct edma_client_internal * get_client_internal(struct edma_client *client);
int32_t create_internal_structures(void);
void free_internal_structures(void);


/* DESCRIPTOR-RELATED FUNCTIONS */
struct edma_descriptor_block * get_descriptor_block(uint32_t channel);
struct edma_descriptor_block * remove_block(uint32_t channel);
struct edma_descriptor_block  * add_new_block(uint32_t channel);
struct edma_descriptor_block * get_last_block(uint32_t channel);
void append_descriptors(uint32_t channel, 
struct edma_descriptor_internal * desc_array);
void edma_descriptors_init(struct edma_chan * chan, 
                           struct edma_descriptor_block  * block);
struct edma_descriptor_internal * get_descriptors(int32_t num, 
                    int32_t channel);



#endif




