/*****************************************************************************
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 *****************************************************************************/
/******************************************************************************
 * @ingroup EDMA_GENERAL
 *
 * @file dma_linux.c
 *
 * @description
 * This file contains initialization and release code, interrupt handler,
 * and tasklet.
 *
 *****************************************************************************/

#ifndef __KERNEL__
#define __KERNEL__
#endif


#include <linux/module.h> /* SET_MODULE_OWNER */
#include <linux/version.h>
#include <linux/sched.h>
#include <linux/kernel.h> /* DMA_DEBUG_PRINTK_0() */
#include <linux/string.h>
#include <linux/init.h>
#include <linux/fs.h>      /* everything... */
#include <linux/errno.h>   /* error codes */
#include <linux/types.h>   /* size_t */
#include <linux/proc_fs.h>
#include <linux/fcntl.h>   /* O_ACCMODE */
#include <asm/system.h>    /* cli, *_flags */
#include <asm/uaccess.h>   /* copy_to_user */
#include <asm/semaphore.h>
#include <asm/io.h>        /* inb(), outb() */
#include <linux/kmod.h>
#include <linux/ioport.h>  /* request_region */
#include <linux/pci.h>
#include <linux/time.h>
#include <linux/interrupt.h> /* tasklets */
#include <linux/list.h>
#include <asm/mman.h>
#include <asm/page.h>
#include <linux/mm.h>
#include <linux/cdev.h> /* For registering the file operations structure */
#include <asm/delay.h>
#include <linux/moduleparam.h>

#include "dma_api.h"


MODULE_AUTHOR("Intel(R) Corporation");
MODULE_DESCRIPTION("Enhanced DMA Controller");
MODULE_LICENSE("Dual BSD/GPL");
MODULE_VERSION("1.0.0");

/*
*  Global variables -
*  Every module is linked to the whole kernel.
*  All symbols that you are not willing to export
*  should be declared static.
*/
/* Initialized elsewhere... */
extern struct edma_descriptor_list g_descriptor_lists[MAX_NO_CHANS];
extern struct edma_chan * g_edma_channel_list[MAX_NO_CHANS];
extern uint32_t g_channel_ccr_offsets[MAX_NO_CHANS];

uint8_t g_drvr_name[] = PRODUCT_STRING;
static struct pci_dev * g_dev_cfg_space = NULL;
struct edma_mm_regs * g_mm_regs;
static struct cdev edma_cdev;
static uint32_t g_task_data[] = { 0, 0, 0, 0};
static spinlock_t g_data_lock;
static uint8_t g_char_drvr_name[] = CHAR_DRV_NAME;
static dev_t dev_no = 0;

static int32_t edma_chan_priority = -1;


module_param(edma_chan_priority, int, S_IRUGO);


/* Prototypes - OS Specific */
int32_t edma_init(void);
void edma_cleanup(void);
int32_t edma_ioctl(struct inode * inode, struct file * filep,
unsigned int cmd, unsigned long arg);
int32_t edma_open(struct inode * inode, struct file * filep);
int32_t edma_release(struct inode * inode, struct file * filep);
int32_t edma_probe(struct pci_dev * dev, const struct pci_device_id * id);
void edma_remove(struct pci_dev * dev);
int32_t edma_suspend (struct pci_dev *dev, pm_message_t state);
int32_t edma_resume(struct pci_dev *dev);
int32_t initialize_edma_device(struct edma_device *device);

static irqreturn_t edma_irq_handler(int32_t irq, void * dev_id,
                                    struct pt_regs * regs);

/* Prototypes - Misc. */

void  status_tasklet_func(unsigned long data);


DECLARE_TASKLET(status_tasklet, status_tasklet_func,
(unsigned long) &g_task_data);

struct file_operations edma_fileops = {
    .owner = THIS_MODULE,
    .open =  edma_open,
    .release = edma_release
     };

static struct pci_device_id pci_ids[] = {
    { PCI_DEVICE(EDMA_VENDOR_ID, EDMA_DEVICE_ID) },
    { 0, },
};

MODULE_DEVICE_TABLE(pci, pci_ids);

static struct pci_driver pci_ops = {
    .name      = g_char_drvr_name,
    .id_table  = pci_ids,
    .probe     = edma_probe,
    .remove    = edma_remove,
    .suspend   = edma_suspend,
    .resume    = edma_resume
};



/* Debug Print Statements */
#ifdef DMA_DEBUG_PRINT
#define DMA_DEBUG_PRINTK_0(S) printk(S )
#define DMA_DEBUG_PRINTK_1(S,A) printk(S,A)
#define DMA_DEBUG_PRINTK_2(S,A,B) printk(S,A,B)
#else
#define DMA_DEBUG_PRINTK_0(S)
#define DMA_DEBUG_PRINTK_1(S,A)
#define DMA_DEBUG_PRINTK_2(S,A,B)
#endif
/*****************************************************************************
 * Abstract
 * Wrapper function that returns memory mapped register.
 * Side Effects:
 * Assumptions:
 * The edma_client_internal structure is not to be confused with edma_client.
 * This function is not part of the client driver API or the application
 * interface.
 * It is called internally.
 * Return Values: Nothing
 *****************************************************************************/
inline struct edma_chan * get_channel(
struct dma_chan * chan)
{
    return (struct edma_chan *)((uint8_t*)chan - \
    (GET_CHANNEL * sizeof(uint8_t *)));

}





/*****************************************************************************
 * Abstract
 * Returns a pointer to a dma_chan struct.
 * Side Effects: Used internally.
 * Assumptions:
 * Return Values: channel ptr or NULL (failed)
 *****************************************************************************/

struct dma_chan * edma_get_channel(void)
{
    struct dma_chan *p_dma_chan = NULL;
    struct edma_device *device = NULL;
    struct edma_chan * p_chan = NULL;
    int32_t found = -1;

    if(g_dev_cfg_space!= NULL)
    {
        device = pci_get_drvdata(g_dev_cfg_space);

        list_for_each_entry(p_dma_chan, &device->common.channels, device_node)
        {
            p_chan = get_channel(p_dma_chan);
            if(p_chan->sw_in_use == NOT_IN_USE)
            {
                p_chan->sw_in_use = IN_USE;
                found = 0;
                break;
            }


        }
    }

    if(found == 0)
    {
       return p_dma_chan;
    }
    else
    {
       return NULL;
    }
}



/*****************************************************************************
 * Abstract
 * Suspend function for power management features of the driver .
 * Side Effects:
 * Assumptions:
 * Return Values: 0 = successful, non-zero unsuccessful.
 *****************************************************************************/
int32_t edma_suspend (struct pci_dev *dev, pm_message_t state)
{
    int32_t result = 0;

    result = edma_suspend_device();
    if(result != 0)
    {
        return result;
    }

    if(g_mm_regs)
    {
        /* free all pci-related resources */
        iounmap(g_mm_regs);
    }


    /*Free IRQ: This was suggested by a team member as a fail-safe.
    It may not be necessary. This function is a void function. */
    free_irq(dev->irq, dev);
    pci_disable_msi(dev);

    /* Cleanup with helpful PCI functions */
    /* Currently, this function always returns 0.*/
    pci_save_state(dev);
    /* pci_disable_device returns nothing - a void function. */
    pci_disable_device(dev);
    /* The pci_set_power_state returns
     * -EINVAL if the device is already in a higher
     *  power state than the value of state being passed.
     *  In this environment, which only goes into standby,
     *  it is going to return a negative value every
     *  time; therefore, I am not checking this value.
     */



    return result;
}

/*****************************************************************************
 * Abstract
 * Resume function for power management features of the driver.
 * Side Effects:
 * Assumptions:
 * Return Values: 0 = successful.
 *****************************************************************************/
int32_t edma_resume(struct pci_dev *dev)
{
    uint32_t resource_start;
    int32_t msi_result;

    pci_enable_device(dev);
    pci_restore_state (dev);
    resource_start = pci_resource_start(dev, 0);
    g_mm_regs = (struct edma_mm_regs *)resource_start;
    g_mm_regs = ioremap((uint32_t)g_mm_regs,
    sizeof(struct edma_mm_regs));

    if(g_mm_regs == NULL)
    {
        DMA_DEBUG_PRINTK_0(ERROR_MM2);
        return -ENODEV;

    }

    if((msi_result = pci_enable_msi(dev)))
    {
        DMA_DEBUG_PRINTK_1(KERN_INFO ERROR_MSI_SUPPORT, msi_result);
        return -ENODEV;

    }
    else
    {
        DMA_DEBUG_PRINTK_1(KERN_INFO ERROR_MSI_VECTOR, dev->irq);


    }

    if(pci_write_config_byte(dev, EDMA_CTL_REG, EDMA_CTL_ENABLE) < 0)
    {
        DMA_DEBUG_PRINTK_0(ERROR_INIT_EDMA_CTL_REG);
        return -ENODEV;
    }
    if(pci_write_config_dword(dev, EDMA_FERR_REG, EDMA_FERR_RESET) < 0)
    {

        DMA_DEBUG_PRINTK_0(ERROR_INIT_EDMA_FERR_REG);
       return -ENODEV;
    }
    if(pci_write_config_word(dev, EDMA_PCICMD, EDMA_CMD_VAL) < 0)
    {


        DMA_DEBUG_PRINTK_0(ERROR_INIT_PCICMD);
        return -ENODEV;
    }

    if (request_irq(dev->irq, &edma_irq_handler, SA_SHIRQ,
                        g_char_drvr_name, dev) )
    {

        DMA_DEBUG_PRINTK_0(ERROR_MM1);
        return -ENODEV;
    }

    edma_resume_device();


    return 0;
}
/*
 *
 *
 * NOTE: The open and release functions were implemented so that
 * the unit test application could have a mechanism for opening and
 * closing the file structure associated with character drivers.
 * The functions are not used by the client driver.
 *
 *
 */

/*****************************************************************************
 * Abstract
 * Open function for testing dma driver as a character driver.
 * Side Effects:
 * Assumptions:
 * This function is currently stubbed out in case the driver is ever used
 * by an application.
 * Prints a message on being called.
 * Return Values: 0 = successful.
 *****************************************************************************/
int32_t edma_open(struct inode * inode,
                  struct file * filep)
{



    pci_write_config_dword(g_dev_cfg_space, EDMA_FERR_REG, EDMA_FERR_RESET);
    /*
    *  The MOD_INC_USE_COUNT is a macro for incrementing the driver
    *  reference count that is used in 2.4.x kernels but is not
    *  used in the 2.6.x kernels.
    */

    return 0;
}
int32_t edma_release(struct inode * inode, struct file * filep)
{
    return 0;
}
/*****************************************************************************
 * Abstract
 * Interrupt handler, called on DMA_ABORTED and END_OF_CHAIN events.
 *
 * Side Effects:
 * Assumptions:
 * Return Values: HANDLED = 1, NOT_HANDLED = 0
 *****************************************************************************/

static irqreturn_t edma_irq_handler(int32_t irq, void * dev_id,
                                    struct pt_regs * regs)
{

    uint32_t clear_bits;
    uint32_t chan_csr_val;
    int32_t chan_cnt = 0;
    int32_t handled = 0;



    /*  TAKE OUT
     *  This is simply not needed here.
    uint32_t flags = 0;
      os_spin_lock_irqsave(&g_data_lock, flags); */


    while(chan_cnt < MAX_NO_CHANS)
    {
        chan_csr_val = readl((uint8_t*)g_mm_regs +
        g_channel_ccr_offsets[chan_cnt] + EDMA_CSR);
        clear_bits =
        chan_csr_val & (END_OF_CHAIN | END_OF_TRANSFER | DMA_ABORTED);
        if(clear_bits != 0)
        {
            writel(clear_bits, (uint8_t*)g_mm_regs +
            g_channel_ccr_offsets[chan_cnt]  + EDMA_CSR);
        }
        if(chan_csr_val & DMA_ABORTED)
        {
            handled = 1;
        }
        if(chan_csr_val & END_OF_CHAIN)
        {
            /* End of chain message */
            /* Debug Mechanism
             * outb(0xEE, 0x80);
             */
            handled = 1;

        }

        g_task_data[chan_cnt] = clear_bits;

        chan_cnt++;

    }
     /*
      *  TAKE OUT
      *  This is simply not needed here.
        os_spin_unlock_irqrestore(&g_data_lock, flags); */


    if(handled == 1)
    {
        tasklet_schedule(&status_tasklet);
    }

    return IRQ_RETVAL(handled);
}

/*****************************************************************************
 * Abstract
 * Bottom half tasklet that does EDMA_ABORT and END_OF_CHAIN cleanup.
 *
 * Side Effects:
 * Assumptions:
 * Return Values: HANDLED = 1, NOT_HANDLED = 0
 *****************************************************************************/
void status_tasklet_func(unsigned long data)
{

    edma_cookie_t cookie = 0;
    struct edma_descriptor_block * block = NULL;
    struct edma_descriptor_internal * desc_array = NULL;
    uint32_t cdar = 0;
    uint32_t cduar = 0;
    uint32_t flags = 0;
    uint32_t chan_cnt = 0;
    unsigned long data_array[] = {0,0,0,0};
    int32_t desc_index = 0;
    int32_t found_index = 0;
    edma_addr_t physical_addr = 0;
    uint32_t ferr_value = 0;
    struct edma_chan * chan = NULL;
    struct edma_descriptor_list * list = NULL;
    int i = 0;
    int last_used_index;


    /* We require an irq spinlock here.*/


    /* SPINLOCK NOTE: Sharing this data with the interrupt handler above.
     * To prevent a HW interrupt from happening while it is being copied,
     * use this style of spinlock.
     */

    os_spin_lock_irqsave(&g_data_lock, flags);
    memcpy(data_array,
    (unsigned long *)data, sizeof(unsigned long) * MAX_NO_CHANS);
    for(i = 0; i < MAX_NO_CHANS; i++)
        g_task_data[i] = 0;
    os_spin_unlock_irqrestore(&g_data_lock, flags);


    for(chan_cnt = 0; chan_cnt < MAX_NO_CHANS; chan_cnt++)
    {
        chan = g_edma_channel_list[chan_cnt];
        list = &g_descriptor_lists[chan_cnt];
        if(data_array[chan_cnt])
        {


             /* Obtain the next descriptor address */
            cdar =
            readl((uint8_t*)g_mm_regs + g_channel_ccr_offsets[chan_cnt] + \
            EDMA_CDAR);
            cduar =
            readl((uint8_t*)g_mm_regs + g_channel_ccr_offsets[chan_cnt] + \
            EDMA_CDUAR);
            physical_addr = cduar;
            physical_addr = physical_addr << SHIFT_DWORD;
            physical_addr = physical_addr | cdar;


            if(data_array[chan_cnt] & END_OF_CHAIN)
            {
                DMA_DEBUG_PRINTK_1(EDMA_END_OF_CHAIN_VAL, chan_cnt);
                cookie = chan->cookie;
                /* SPINLOCK NOTE: Sharing this data (descriptor blocks) with
                 * the rest of the driver.
                 * To prevent a SW level interrupt from happening, while the
                 * list is being processed, use this style of spinlock.
                 */
                os_spin_lock_irq(&chan->desc_lock);
                block = (struct edma_descriptor_block *)
                list->block_list.next_ptr;

                do
                {


                    /* Correct the head and tail indices,
                     * for the block.
                     */

                    desc_array = block->descriptor_pool;
                    last_used_index =  block->free_index - 1;

                    /* Looking for last completed cookie */
                    /* Write completed cookie information */
                    /* If this condition true, the block is found */
                    if((physical_addr >= desc_array[0].phys)  &&
                    (physical_addr <= \
                    desc_array[DEFAULT_DESCRIPTOR_NO - 1].phys))
                    {
                        block->used_index = last_used_index;

                        for(i = 0; i <= (DEFAULT_DESCRIPTOR_NO - 1); i++)
                        {
                            if(desc_array[i].phys == physical_addr)
                            {
                                DMA_DEBUG_PRINTK_1 ("COOKIE_RESOLVED %x \n",
                                (int) desc_array[i].cookie);
                                chan->completed_cookie = desc_array[i].cookie;
                                break;
                            }
                        }

                    }

                    /* Block was not found, but it is obsolete. */
                    else
                    {
                        if(block->next_ptr != NULL)
                        {
                             block->status = OBSOLETE;
                             DMA_DEBUG_PRINTK_1(
                             "MARK_OBSOLETE number block %x\n",
                             (int)block);
                        }
                    }

                    block = (struct edma_descriptor_block *)
                    block->next_ptr;

                }while(block != NULL);
                os_spin_unlock_irq(&chan->desc_lock);
            }

            if(data_array[chan_cnt] & DMA_ABORTED)
            {


                pci_read_config_dword(g_dev_cfg_space,
                EDMA_FERR_REG, &ferr_value);
                pci_write_config_dword(g_dev_cfg_space, EDMA_FERR_REG, ~0);
                DMA_DEBUG_PRINTK_1(EDMA_ABORTED_FERR_VAL, ferr_value);

               /* SPINLOCK NOTE: Sharing this data (descriptor blocks) with the
                * rest of the driver.
                * To prevent a SW level interrupt from happening, while the
                * list is being processed, use this style of spinlock.
                */
                os_spin_lock_irq(&chan->desc_lock);
                block = (struct edma_descriptor_block *)
                list->block_list.next_ptr;

                desc_array = block->descriptor_pool;
                /* Correct the used and free indices,
                 * for the block.
                */
                /* Find the problematic descriptor */
                /* Check for an EOC */
                /* Not an EOC, need to reroute and restart engine. */
                do
                {


                    desc_index = 0;
                    found_index = -1;

                    while(desc_index != block->free_index)
                    {

                        if(desc_array[desc_index].phys == physical_addr)
                        {

                            block->used_index = found_index + 1;
                            block->last_ferr_value = ferr_value;
                            block->last_cookie_err =
                            desc_array[desc_index].cookie;
                            block->last_desc_err = desc_index;
                            found_index = desc_index;

                            break;
                        }
                        else
                        {
                           desc_index++;
                        }

                    }

                    if(found_index != -1)
                    {

                        if(desc_array[found_index].hw.ndar == 0
                        && desc_array[found_index].hw.nduar == 0)
                        {
                           break;
                        }
                        else
                        {

                           if(desc_array[found_index + 1].cookie != -1)
                           {
                               writel(
                               desc_array[found_index + 1].phys & LOWER_64,
                               (uint8_t *)g_mm_regs + \
                               g_channel_ccr_offsets[chan_cnt]
                               + EDMA_NDAR);
                               writel(
                               (desc_array[found_index + 1].phys >> \
                               SHIFT_DWORD) & LOWER_64,
                               (uint8_t *)g_mm_regs + \
                               g_channel_ccr_offsets[chan_cnt]
                               + EDMA_NDUAR);
                               writel(START_DMA,
                               (uint8_t *) g_mm_regs + \
                               g_channel_ccr_offsets[chan_cnt]);

                               block->used_index = block->free_index;

                               break;
                           }


                        }


                    }
                    if(block->next_ptr != NULL)
                    {

                         block = (struct edma_descriptor_block *)
                         block->next_ptr;
                         desc_array = block->descriptor_pool;

                    }
                    else
                    {
                         break;
                    }


                }while(1);
                os_spin_unlock_irq(&chan->desc_lock);




            }
        }
    }



}

/*****************************************************************************
 * Abstract
 * Probe is called upon loading the driver, initializes the driver's internal
 * data structures, allocates irq, mallocs memory, etc.
 *
 * Side Effects:
 * Assumptions:
 * Return Values: SUCCESS (0), anything else equals a failure to successfully
 * initialize edma.
 *****************************************************************************/
int32_t edma_probe(struct pci_dev * dev, const struct pci_device_id * id)
{

    int32_t msi_result  = 0;
    uint32_t resource_start;
    struct edma_device * device =
    kmalloc(sizeof(struct edma_device), GFP_KERNEL);
    g_dev_cfg_space = dev;


    DMA_DEBUG_PRINTK_0(MESSAGE_PROBE);

    if(pci_enable_device(dev))
    {
        DMA_DEBUG_PRINTK_0(ERROR_DEVICE_NOT_FOUND);
        return -ENODEV;

    }

    if(pci_write_config_byte(dev, EDMA_CTL_REG, EDMA_CTL_ENABLE) < 0)
    {
        DMA_DEBUG_PRINTK_0(ERROR_INIT_EDMA_CTL_REG);
        return -ENODEV;
    }
    if(pci_write_config_dword(dev, EDMA_FERR_REG, EDMA_FERR_RESET) < 0)
    {

        DMA_DEBUG_PRINTK_0(ERROR_INIT_EDMA_FERR_REG);
       return -ENODEV;
    }
    if(pci_write_config_word(dev, EDMA_PCICMD, EDMA_CMD_VAL) < 0)
    {


        DMA_DEBUG_PRINTK_0(ERROR_INIT_PCICMD);
        return -ENODEV;
    }

    if(pci_request_region(dev, EDMA_BAR0, g_char_drvr_name) == SUCCESS)
    {
        /*
         *  Obtain a virtual address for the memory mapped registers.
         *  The ioremap function makes bus memory CPU accessible via
         *  the readb/readw/readl/writeb/writew/writel functions.
         */
          resource_start = pci_resource_start(dev, 0);
          g_mm_regs = (struct edma_mm_regs *)resource_start;
          g_mm_regs = ioremap((uint32_t)g_mm_regs,
          sizeof(struct edma_mm_regs));

          if(g_mm_regs == NULL)
          {
            DMA_DEBUG_PRINTK_0(ERROR_MM2);
            return -ENODEV;

          }

     }
     else
     {
        DMA_DEBUG_PRINTK_0(ERROR_MM2);
        return -ENODEV;
     }

    if((msi_result = pci_enable_msi(dev)))
    {
        DMA_DEBUG_PRINTK_1(KERN_INFO ERROR_MSI_SUPPORT, msi_result);

    }
    else
    {
        DMA_DEBUG_PRINTK_1(KERN_INFO ERROR_MSI_VECTOR, dev->irq);

    }


    /*
    * Obtain a (shared) Interrupt Request (IRQ) Line from the OS.
    */
    if (request_irq(dev->irq, &edma_irq_handler, SA_SHIRQ,
                        g_char_drvr_name, dev) )
    {

        DMA_DEBUG_PRINTK_0(ERROR_MM1);
        return -ENODEV;
    }

    device->pci_dev = dev;
    pci_set_drvdata(dev, device);
    edma_change_channel_priority(edma_chan_priority);


    if(create_internal_structures())
    {
        DMA_DEBUG_PRINTK_0(ERROR_INTERNAL_STRUCT);

    }




    return SUCCESS;
}

/*****************************************************************************
 * Abstract
 * Called during unloading.
 *
 * Side Effects:
 * Assumptions:
 * Return Values: None
 *****************************************************************************/
void edma_remove(struct pci_dev * dev)
{
    struct edma_device * device = pci_get_drvdata(dev);

    DMA_DEBUG_PRINTK_0(MESSAGE_REMOVE);
    free_internal_structures();

    /* free the IRQ line */
    free_irq(dev->irq, dev);
    pci_disable_msi(dev);

    if(g_mm_regs)
    {
        /* free all pci-related resources */
        iounmap(g_mm_regs);
    }

    pci_release_region(dev, 0);
    pci_disable_device(dev);
    kfree(device);

}
/*****************************************************************************
 * Abstract
 * This function is probes for the hardware device, and it intializes the
 * devices, sets up the interrupt handler, sets up the memory mapped
 * registers, etc.
 *
 * Side Effects:
 * Assumptions:
 * Prints a message on being called.
 * Return Values: <=0  successful, positive value is unsuccessful.
 *****************************************************************************/
int32_t edma_init(void)
{

    /*
     * Initialize dev, NULL indicates that pci_find_device should
     * start at the beginning to look for the device.
     */
    int32_t ret         = 0;
    g_mm_regs       = NULL;

    DMA_DEBUG_PRINTK_0(KERN_INFO MESSAGE_INIT);

    cdev_init(&edma_cdev, &edma_fileops);
    edma_cdev.ops = &edma_fileops;



    if(alloc_chrdev_region(&dev_no, 0, 1, g_char_drvr_name) != 0)
    {
        DMA_DEBUG_PRINTK_0(CHAR_DVR_REG_FAILED);
        goto Exit_Error;
    }
    if(cdev_add(&edma_cdev, dev_no,  1) != 0)
    {
        DMA_DEBUG_PRINTK_1(CHAR_DEVICE_ALLOC, dev_no);
        goto Exit_Error;
    }

    /*
    * Create internal structures, which involves several kmallocs.
    * If the kmalloc calls fail, go to Exit_Error.
    */
    spin_lock_init(&g_data_lock);

    if(pci_register_driver(&pci_ops) < 0)
    {

        DMA_DEBUG_PRINTK_0(ERROR_PCI_REGISTRATION);
        goto Exit_Error;

    }

    goto Exit;


/* A serious error occurred. */
Exit_Error:
    edma_cleanup();
    ret = -ENODEV;

Exit:
    DMA_DEBUG_PRINTK_0(MESSAGE_INIT_COMPLETE);
    return ret;

}


/*****************************************************************************
 * Abstract
 * This function undoes everything done in the init function including
 * tearing down the interrupt handler.
 *
 * Side Effects:
 * Assumptions:
 * Prints a message on being called.
 * Return Values: NONE
 *****************************************************************************/
void edma_cleanup(void)
{
   /*
    *  Print a kernel message acknowledging the unloading of the driver
    */
    DMA_DEBUG_PRINTK_0(KERN_INFO MESSAGE_CLEANUP);



    /* If the unregister_chrdev fails, it is a serious problem,
     * as the next invocation of insmod will produce errors.
     */

    cdev_del(&edma_cdev);
    unregister_chrdev_region(dev_no, 1);
    pci_unregister_driver(&pci_ops);
}


EXPORT_SYMBOL(edma_get_functions);

module_init(edma_init);
module_exit(edma_cleanup);









