/*****************************************************************************
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 *****************************************************************************/

/*****************************************************************************
 * os.h
 *
 * Common Driver OS Portability Enabling Module
 *
 * Description
 *   This is an internal API that intended as a lightweight common 
 *   portability layer that allows driver versions to reduce their 
 *   OS dependent code. 
 *
 *****************************************************************************/

#ifndef __OS_H__
#define __OS_H__

#include "os_types.h"

/******************************************************************************
 * Allocate coherent memory for DMA.
 *****************************************************************************/
void *os_malloc_coherent(void * device, size_t size,
                             int32_t * dma_handle, int32_t gfp);

/******************************************************************************
 * De-allocate coherent memory for DMA.
 *****************************************************************************/
void os_free_coherent(void * device, size_t size,
                             void * dma,int32_t * dma_handle);


/******************************************************************************
 * Allocate memory.
 *****************************************************************************/
void *os_malloc(
	unsigned int		size,
	void			*memory_type,
	unsigned int		flags);

/******************************************************************************
 * Free memory.
 *****************************************************************************/
void os_free(
	void			*mem_ptr,
	void			*memory_type);

/******************************************************************************
 * Lock a spinlock.
 *****************************************************************************/
void os_spin_lock(os_spinlock_t *spin);

/******************************************************************************
 * Unlock a spinlock.
 *****************************************************************************/
void os_spin_unlock(os_spinlock_t *spin);

/******************************************************************************
 * Initialize a spinlock.
 *****************************************************************************/
void os_spin_lock_init(os_spinlock_t *spin);

/******************************************************************************
 * Lock a spinlock.
 *****************************************************************************/
void os_spin_lock_irq(os_spinlock_t *spin);

/******************************************************************************
 * Unlock a spinlock.
 *****************************************************************************/
void os_spin_unlock_irq(os_spinlock_t *spin);

/******************************************************************************
 * Lock a spinlock.
 *****************************************************************************/
void os_spin_lock_bh(os_spinlock_t *spin);

/******************************************************************************
 * Unlock a spinlock.
 *****************************************************************************/
void os_spin_unlock_bh(os_spinlock_t *spin);

/******************************************************************************
 * Save IRQ
 *****************************************************************************/
void os_spin_lock_irqsave(os_spinlock_t *spin, uint32_t flags);

/******************************************************************************
 * Restore IRQ
 *****************************************************************************/
void os_spin_unlock_irqrestore(os_spinlock_t *spin, uint32_t flags);

/******************************************************************************
 * Virtual memory to physical.
 *****************************************************************************/
unsigned int os_virt_to_phys(void *ptr);

/******************************************************************************
 * Physical memory to virutal.
 *****************************************************************************/
unsigned int os_phys_to_virt(unsigned int address);
	
/******************************************************************************
 * Write long register.
 *****************************************************************************/
void os_writel(
	unsigned int		val,   
	void			*addr);

/******************************************************************************
 * Write word register.
 *****************************************************************************/
void os_writew(
	unsigned short		val, 
	void			*addr);

/******************************************************************************
 * Write byte register.
 *****************************************************************************/
void os_writeb(
	unsigned char		val,  
	void			*addr);

/******************************************************************************
 * Read long register.
 *****************************************************************************/
unsigned int os_readl(void *addr);

/******************************************************************************
 * Read word register.
 *****************************************************************************/
unsigned int os_readw(void *addr);

/******************************************************************************
 * Read byte register.
 *****************************************************************************/
unsigned int os_readb(void *addr);

/******************************************************************************
 * Write out to a port.
 ******************************************************************************/
void os_outb(uint32_t port, uint32_t value);

#endif /* __OS_H__ */
