/*****************************************************************************
 * This file is provided under a dual BSD/GPLv2 license.  When using or 
 *   redistributing this file, you may do so under either license.
 * 
 *   GPL LICENSE SUMMARY
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 * 
 *   This program is free software; you can redistribute it and/or modify 
 *   it under the terms of version 2 of the GNU General Public License as
 *   published by the Free Software Foundation.
 * 
 *   This program is distributed in the hope that it will be useful, but 
 *   WITHOUT ANY WARRANTY; without even the implied warranty of 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
 *   General Public License for more details.
 * 
 *   You should have received a copy of the GNU General Public License 
 *   along with this program; if not, write to the Free Software 
 *   Foundation, Inc., 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *   The full GNU General Public License is included in this distribution 
 *   in the file called LICENSE.GPL.
 * 
 *   Contact Information:
 *   Intel Corporation
 * 
 *   BSD LICENSE 
 * 
 *   Copyright(c) 2007,2008 Intel Corporation. All rights reserved.
 *   All rights reserved.
 * 
 *   Redistribution and use in source and binary forms, with or without 
 *   modification, are permitted provided that the following conditions 
 *   are met:
 * 
 *     * Redistributions of source code must retain the above copyright 
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright 
 *       notice, this list of conditions and the following disclaimer in 
 *       the documentation and/or other materials provided with the 
 *       distribution.
 *     * Neither the name of Intel Corporation nor the names of its 
 *       contributors may be used to endorse or promote products derived 
 *       from this software without specific prior written permission.
 * 
 *   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
 *   "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
 *   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
 *   A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
 *   OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT 
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, 
 *   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY 
 *   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT 
 *   (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE 
 *   OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 * 
 *  version: Embedded.L.1.0.34
 *****************************************************************************/

/*****************************************************************************
 * os_list.h         
 *
 * Linked list data structure.
 * 
 * Description
 *      These functions create/destroy, manipulate a linked-list.
 * 
 *****************************************************************************/

#ifndef __OS_LIST_H__
#define __OS_LIST_H__

/******************************************************************************
 * Inserts a list element at the end of the list.
 *****************************************************************************/
struct os_list_start
{
    unsigned char *next_ptr; /* ptr to the next item in the list */
    unsigned char *prev_ptr; /* ptr to previous item in list */ 
};

/******************************************************************************
 * Inserts a list element at the end of the list.
 *****************************************************************************/
void os_list_insert_element_at_end(
	struct os_list_start	*element, 
	struct os_list_start	*head);

/******************************************************************************
 * Adjusts the pointers when a list element is removed by
 * the removal functions.
 *****************************************************************************/
void os_list_adjust_ptrs(
	struct os_list_start	*prev, 
	struct os_list_start	*head);

/******************************************************************************
 * Removes an element from a list.
 *****************************************************************************/
void os_list_remove_element(
	struct os_list_start	*element, 
	struct os_list_start	*head);

/******************************************************************************
 * Initializes list to proper empty values.
 *****************************************************************************/
void os_list_linked_list_reinit(struct os_list_start *list);

/******************************************************************************
 * Removes an element from the front of the list.
 *****************************************************************************/
unsigned char *os_list_remove_each_element_front(struct os_list_start *head);

/******************************************************************************
 * Inserts a list element at the front of the list.
 *****************************************************************************/
void os_list_insert_element_front(
	struct os_list_start	*element, 
	struct os_list_start	*head);

#endif /* __OS_LIST_H__ */
